/* FILE: diag.c                 Copyright (c), Symmetric Research, 1998-2000
 *
 * This diagnostic program checks for proper installation of the SR PAR24B
 * board.
 *
 * If successful, the user can twiddle the yellow LED.
 *
 *
 */

#include <stdio.h>
#include "par24b.h"

#ifdef SROS_LINUX
#include <unistd.h>
#define flushprint(); fflush( STDIN_FILENO );
#else
#define flushprint(); 
#endif







/* DISPLAY HELPER FUNCTIONS:
 *
 *
 */

char *Chop( char *s ) {

        char *save;


        /* Chop white space off the end of a string. */

        save = s;

        while ( *s )

                s++;

        if ( *--s == '\n' ) {

                while ( *--s == ' ' )

                        /* back up */ ;

                *++s = '\n';
                *++s = '\0';

                }

        return( save );
}

void PrintMsg( char *MsgArray[] ) {

        int i;


        /* Print the message array, chopping trailing white space. */

        for ( i = 0 ; MsgArray[i] ; i++ )

                printf( Chop( MsgArray[i] ) );
}

char *StrPortMode( int value ) {


        /* Convert the PortMode flag into a string. */

        if ( value == PAR24B_PORT_MODE_BPP )

                return( "    BPP");

        else if ( value == PAR24B_PORT_MODE_EPP )

                return( "    EPP");

        else if ( value == PAR24B_PORT_MODE_ECP_BPP )

                return( "ECP/BPP" );

        else if ( value == PAR24B_PORT_MODE_ECP_EPP )

                return( "ECP/EPP" );

        else
                return( "UNKNOWN" );

}









/* DIAGNOSTIC MESSAGES:
 *
 *
 *
 *
 */

char *LEADER[] = {

    "PAR24B diag : (c) SR 1999 : Rev {12/15/01}                           \n",
    "                                                                     \n",

    NULL,

    };


char *NOT_FOUND[] = {

    "Failed to find PAR24B at any Port address or mode ...                \n",
    "                                                                     \n",
    "Possible problems:                                                   \n",
    "                                                                     \n",
    " * Is a board connected and powered up ?                             \n",
    " * Remove/disable a competing printer driver (HP etc)                \n",
    " * Check Win 95 Plug and Play.  Reserve desired port address.        \n",
    " * Is Win NT or Linux device driver installed?                       \n",
    "                                                                     \n",

    NULL,

    };








/* TEST DATA:
 *
 *
 *
 *
 */

#define SUCCESS         1
#define FAILURE         0


struct test_scheme

   { int PortAddress, PortMode, Status; }

        TestScheme[] = {

                PAR24B_PORT_ADDRESS_1, PAR24B_PORT_MODE_EPP,     FAILURE,
                PAR24B_PORT_ADDRESS_1, PAR24B_PORT_MODE_BPP,     FAILURE,
                PAR24B_PORT_ADDRESS_1, PAR24B_PORT_MODE_ECP_EPP, FAILURE,
                PAR24B_PORT_ADDRESS_1, PAR24B_PORT_MODE_ECP_BPP, FAILURE,

                PAR24B_PORT_ADDRESS_2, PAR24B_PORT_MODE_EPP,     FAILURE,
                PAR24B_PORT_ADDRESS_2, PAR24B_PORT_MODE_BPP,     FAILURE,
                PAR24B_PORT_ADDRESS_2, PAR24B_PORT_MODE_ECP_EPP, FAILURE,
                PAR24B_PORT_ADDRESS_2, PAR24B_PORT_MODE_ECP_BPP, FAILURE,

                0x000,

                };





/* MAIN:
 *
 *
 *
 *
 *
 */

int main( void ) {

        long int i, status, ibest, HI7190_CrValue;

            unsigned int FifoSize;

                int LedValue, ErrorCode;




        /* Display the leader message. */

        PrintMsg( LEADER);





        /* Test for the PAR24B at various addresses and modes. */

        ibest = -1;

        for ( i = 0 ; TestScheme[i].PortAddress ; i++ ) {



                /* Display the Port being tested ... */

                printf( "Testing  0x%X, %s ... ",

                                             TestScheme[i].PortAddress,
                                StrPortMode( TestScheme[i].PortMode)

                                );  flushprint();



                /* Do a presence test. */

#define SKIP_FIFO_SIZE          NULL
#define IGNORE_ERRORCODES       NULL

                status = PAR24B_Init(

                                 TestScheme[i].PortAddress,
                                 TestScheme[i].PortMode,

                                 SKIP_FIFO_SIZE,

                                 PAR24B_DC_SIGNED,
                                 PAR24B_FP_30Hz,
                                 PAR24B_GAIN_1,
                                 PAR24B_CAL_MODE_SELF,

                                 &HI7190_CrValue,
                                     
                                 &ErrorCode

                                 );



                TestScheme[i].Status = status;

                if ( status == 1 )

                        printf( "status = SUCCESS\n");

                else
                        printf( "status = failure %s\n",
                                                     FAILURE_WORD[ErrorCode] );




                /* Save the index of the first successful scheme. */

                if ( TestScheme[i].Status == SUCCESS && ibest == -1 )

                        ibest = i;

                }


        printf( "\n");






        /* If no (address,mode) succeeded, give help and quit. */

        if ( ibest == -1 )

                { PrintMsg( NOT_FOUND); PAR24B_Close(); exit( 1); }






        /* Reinitialize the best (address,mode) and compute the FIFO size. */

        printf( "Best port address = 0x%X, mode = %s, initializing ... ",

                                             TestScheme[ibest].PortAddress,
                                StrPortMode( TestScheme[ibest].PortMode)

                                );  flushprint();

        status = PAR24B_Init(

                 TestScheme[ ibest].PortAddress,
                 TestScheme[ ibest].PortMode,

                 &FifoSize,

                 PAR24B_DC_SIGNED,
                 PAR24B_FP_30Hz,
                 PAR24B_GAIN_1,
                 PAR24B_CAL_MODE_SELF,

                 &HI7190_CrValue,

                 &ErrorCode

                 );


        printf( " done\n\n");


        if ( status == 0 )

                printf( "status = failure %s\n\n", FAILURE_WORD[ErrorCode] );



        /* Report the FIFO size. */

        printf( "PAR24B Fifo size = %u bytes\n\n", FifoSize);





        /* Let them twiddle the yellow LED. */

        printf( "Hit ENTER to twiddle the yellow LED, q+ENTER quits ... \n\n");

        LedValue = 0;

        while (1) {

                PAR24B_UserByte( LedValue);

                printf( "Yellow LED = %1d ", LedValue);  flushprint();

                LedValue ^= 1;

                if ( GetKeyWait() == 'q' )
                        break;
                        
                }



        PAR24B_Close();

        
        return( 0 );
}
