/* FILE: sample.c               Copyright (c), Symmetric Research, 1998-1999
 *
 * This is a simple bare bones program to demonstrate running the PAR24B
 * board with the par24b function library.  It has the minimum code required
 * to run the board.
 *
 * Note that the library must be located along the library search path
 * before sample will run.  See readme.txt for how to set the library path.
 *
 * If you want to change any of the sample.c initialization parameters,
 * you must change its source code and recompile.  See the makefile for
 * how to compile.
 *
 * Simp24b.c provides a more flexible interface for changing the init
 * parameters.  Use this program for experimenting with programming the
 * PAR24B in C.
 *
 *
 *
 *
 *
 */

#include <stdio.h>
#include <malloc.h>

#include "par24b.h"








/* DISPLAY and ERROR HELPER FUNCTION:
 *
 *
 *
 *
 */

#define PRNT_FMT  "    %08lX %08lX %08lX %08lX %08lX %08lX %08lX %08lX\n"


void DisplayValues( long *Values, unsigned int Nvalues) {

        int i;

        printf( "PAR24B data ... \n");

          for ( i = 0 ; i < Nvalues ; i += 8 )

                printf( PRNT_FMT,

                        Values[i+0],
                        Values[i+1],
                        Values[i+2],
                        Values[i+3],
                        Values[i+4],
                        Values[i+5],
                        Values[i+6],
                        Values[i+7]

                        );

        printf( "\n");

}
void Error( char *msg) {


        /* Print an error message and quit. */

        printf( "ERROR: %s\n", msg);

        exit( 1);

}







/* MAIN:
 *
 *
 *
 *
 */

int main( void) {

        long *PC_BUFFER; unsigned int FifoSize, Nvalues;


#ifdef SROS_LINUX

        /* Allow getchar to return even if a key has not been pressed. */

        if ( SetNonBlock( 1 ) == -1 )
               Error( "Can't set non-blocking read ...");
#endif


        /* Initialize the PAR24B.
         *
         * The defined constants used with PAR24B_Init come from par24b.h.
         *
         *
         */

        if ( !PAR24B_Init(

                        PAR24B_PORT_ADDRESS_1,
                        PAR24B_PORT_MODE_BPP,

                        &FifoSize,

                        PAR24B_DC_SIGNED,
                        PAR24B_FP_30Hz,
                        PAR24B_GAIN_1,
                        PAR24B_CAL_MODE_SELF,

                        NULL,
                        NULL

                        )

           ) Error( "PAR24B_Init failed ...");





        /* PC DATA BUFFER:
         *
         * Values from the PAR24B board are saved in this array.
         * 
         * Each 24 bit value is saved as a 32 bit long integer.  Values from
         * the eight channels are saved in multiplexed fashion.
         *
         * The parameter Nvalues is the total number of longs that are
         * ready when PAR24B_IsReady returns true.  See par24b.h for details
         * on this constant.
         *
         *
         *
         *
         */

        PAR24B_FifoSizeToNvalues( FifoSize, &Nvalues);

            PC_BUFFER = (long *)malloc( Nvalues*sizeof( long));






        /* Start executing and wait for data to become ready.
         *
         * NOTE: PC_BUFFER[] MUST be dimensioned to PAR24B_NVALUES.
         *
         *
         *
         */

        printf( "Simple PAR24B example, press ENTER to quit ...\n");

        PAR24B_Start();




polling_loop:


        if ( PAR24B_IsReady() ) {


                /* Check for possible FIFO overflow. */

                if ( PAR24B_Overflow() )

                        Error( "FIFO Overflow ...");



                /* Read and display the data from the FIFO. */

                PAR24B_GetData( PC_BUFFER, Nvalues);

                    DisplayValues( PC_BUFFER, Nvalues);


                }





        /* Check for a kbhit to quit. */

        if ( GetKeyCheck() != EOF ) {

                printf( "Kbhit ... quitting\n");

                PAR24B_Stop();

                exit( 0);

                }




        /* Continue polling. */

        goto polling_loop;

}
