/* FILE: par24b.h               Copyright (c), Symmetric Research, 1998-2001
 *
 * SR PAR24B low level/DLL interface library include file.
 *
 *
 *
 */


/* Set up some OS specific values
 *
 *
 */

#if defined( SROS_WIN2K ) || defined( SROS_WINNT )
// None needed

#elif defined( SROS_WIN9X ) || defined( SROS_MSDOS )
typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;

#elif defined( SROS_LINUX )
int  SetNonBlock( int value );
int  _commit( int fh );
int _filelength( int fh );
typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;

#else
#pragma message( "COMPILE ERROR: SROS_xxxxx MUST BE DEFINED !! " )

#endif  // SROS_WIN2K  SROS_WINNT  SROS_WIN9X  SROS_MSDOS  SROS_LINUX





/* FUNCTION AND VARIABLE TYPES:
 *
 * These defines control the function and variable types for the SR PAR24B
 * library.
 *
 * Whether a function is marked for static, DLL export, or DLL import use
 * is controlled by the VISIBILITY define.  With the MS VC 6.0 compiler
 * some acceptable values are:
 *
 *      #define VISIBILITY
 *      #define VISIBILITY      __declspec( dllexport)
 *      #define VISIBILITY      __declspec( dllimport) 
 *
 *
 * The calling convention is controlled by the CALLCONV define.  With the
 * MS VC 6.0 compiler some acceptable values are:
 *
 *      #define CALLCONV
 *      #define CALLCONV        __stdcall
 *
 *
 * A null VISIBILITY string indicates standard static usage.  A null
 * CALLCONV string indicates C calling convention.
 *
 * We assume most users will want to use the library as a DLL from a
 * variety of languages, and so have defined dllimport and stdcall as
 * the defaults below.
 *
 * Users needing to override these defaults should give suitable defines
 * from inside their source code or the compiler command line.
 *
 *
 *
 */

#define VOID                    void
#define INT                     int
#define UINT                    unsigned int
#define I16                     short
#define I32                     long
#define U16                     unsigned short
#define U32                     unsigned long


#ifdef SROS_LINUX
#define _STATIC 1
#undef _DLLEXPORT
#undef _DLLIMPORT
#endif


#ifdef _STATIC
#define VISIBILITY
#define CALLCONV
#endif

#ifdef _DLLEXPORT
#define VISIBILITY              __declspec( dllexport)
#define CALLCONV                __stdcall
#endif

#ifdef _DLLIMPORT
#define VISIBILITY              __declspec( dllimport)
#define CALLCONV                __stdcall
#endif

#ifndef VISIBILITY
#define VISIBILITY              __declspec( dllimport)
#endif

#ifndef CALLCONV
#define CALLCONV                __stdcall
#endif


#define VARTYPE( type)          VISIBILITY type
#define FUNCTYPE( type)         VISIBILITY type CALLCONV







/* RELEASE REV INFO:
 *
 * This define, variable, and function allow the user to determine
 * library rev at either compile or runtime.
 *
 * The rev is an integer, where a number like 134 would be referred to
 * as rev "1.34".
 *
 * Short rev history:
 * 
 *   PAR24B_REV  230  ( 12/20/2000 )  < Toshiba BPP fix
 *   PAR24B_REV  231  ( 12/15/2001 )  < ACPI, 64K FIFO, ECP support
 *   PAR24B_REV  232  ( 12/25/2001 )  < DbEndPtr update fix
 *   PAR24B_REV  233  ( 12/30/2001 )  < Add VB demo, convert
 *   PAR24B_REV  234  ( 01/10/2003 )  < Improve overflow recovery
 *
 */

#define PAR24B_REV              234

extern VARTYPE( INT) PAR24B_Rev;

FUNCTYPE( INT) PAR24B_GetRev( void);



/* Keyboard helper functions */

#define KEY_CHECK 0
#define KEY_WAIT  1

FUNCTYPE( INT) GetKey( int wait );
FUNCTYPE( INT) GetKeyCheck( void );
FUNCTYPE( INT) GetKeyWait(  void );







/* PAR24B_Init DEFINES:
 *
 * Use these defines with PAR24B_Init to initialize the PAR24B board.
 *
 * Several of these constants map directly into HI7190 register bit
 * setings.  Don't change their values!
 *
 * Some of the abbreviations are:
 *
 *       EPP              enhanced parallel port
 *       BPP              bidirectional parallel port
 *       ECP              extended capabilities port
 *
 *       DC               HI7190 data coding
 *       FP               HI7190 filter point (sampling rate)
 *       GAIN             HI7190 preamp gain
 *       CAL_MODE         HI7190 calibration mode
 *
 *
 *
 */


/* parallel port defines ... */

#define PAR24B_PORT_ADDRESS_1           0x378
#define PAR24B_PORT_ADDRESS_2           0x278

#define PAR24B_PORT_MODE_BPP            0
#define PAR24B_PORT_MODE_EPP            1
#define PAR24B_PORT_MODE_ECP_BPP        2
#define PAR24B_PORT_MODE_ECP_EPP        3



/* HI7190 control register defines ... */

#define PAR24B_DC_OFFSET                0
#define PAR24B_DC_SIGNED                1

#define PAR24B_FP_MIN                   10
#define PAR24B_FP_MAX                   2047
#define PAR24B_FP_30Hz                  651      /* = 0x28B ->  30.00192 Hz */
#define PAR24B_FP_100Hz                 195      /* = 0x0C3 -> 100.16025 Hz */

#define PAR24B_GAIN_1                   0
#define PAR24B_GAIN_2                   1
#define PAR24B_GAIN_4                   2
#define PAR24B_GAIN_8                   3
#define PAR24B_GAIN_16                  4
#define PAR24B_GAIN_32                  5
#define PAR24B_GAIN_64                  6
#define PAR24B_GAIN_128                 7

#define PAR24B_CAL_MODE_NONE            0
#define PAR24B_CAL_MODE_SELF            1
#define PAR24B_CAL_MODE_SOIG            5







/* PAR24B_Init ERRORCODES:
 *
 * PAR24B_Init returns a 1 or 0 for success or failure.  If it fails, the
 * errorcode will be filled in with one of the following integers for more
 * information.
 *
 *
 */

#define PAR24B_IERROR_SUCCESS                                   0
#define PAR24B_IERROR_PORT_ADDRESS                              1
#define PAR24B_IERROR_PORT_MODE                                 2
#define PAR24B_IERROR_CANT_COMPUTE_FIFO_SIZE                    3
#define PAR24B_IERROR_DC_OFFSET                                 4
#define PAR24B_IERROR_FP                                        5
#define PAR24B_IERROR_GAIN                                      6
#define PAR24B_IERROR_CAL_MODE                                  7
#define PAR24B_IERROR_HI7190_CR_READBACK                        8
#define PAR24B_IERROR_DRIVER_NOT_OPEN                           9
// If you add errors, be sure to update the FAILURE_WORD strings in par4ch.c

extern char *FAILURE_WORD[];






/* PAR24B SYSTEM DEFINES:
 *
 * The PAR24B is fixed at eight channels.
 *
 * The min and max FIFO sizes are the extremes of commercially available
 * parts in the 72XX class that can be used on the board.
 * 
 * Use PAR24B_Init to dynamically compute the FIFO size on your particular
 * board.
 *
 * Most PAR24B boards ship with a 4K or 32K FIFO, but 64K FIFOs can be
 * purchased directly from chip distributors.  However, the DOS version
 * of this software can not support the 64K FIFO due to limitations
 * imposed by the segemented nature of that OS.
 *
 */

#define PAR24B_CHANNELS                 8

#define PAR24B_FIFO_SIZE_MIN            256          /* 256 bytes    */
#define PAR24B_FIFO_SIZE_TYP            4096         /*  4K bytes    */


#if defined( SROS_MSDOS )
#define PAR24B_FIFO_SIZE_MAX            32768        /* 32K bytes    */
#else
#define PAR24B_FIFO_SIZE_MAX            65536        /* 64K bytes    */
#endif





/* PAR24B CONTROL/EXECUTION FUNCTIONS:
 *
 * These functions control the operation of the PAR24B.
 *
 * Use the defines above and helper functions below for setting PAR24B_Init
 * parameters.
 *
 * The size of the values array to use with PAR24B_GetData, Nvalues, should
 * be computed from the FifoSize with PAR24B_FifoSizeToNvalues.
 *
 * Note that PAR24B_UserByte will not work until the board has been
 * initialized with PAR24B_Init.
 *
 *
 *
 */

FUNCTYPE( INT) PAR24B_Init(

             int PortAddress,
             int PortMode,
    unsigned int *FifoSize,

             int Dc,
             int Fp,
             int Gain,
             int CalMode,
            long *HI7190_CrValue,

             int *ErrorCode
        );

FUNCTYPE( VOID) PAR24B_SpsToFp( double Sps, int *Fp);
FUNCTYPE( VOID) PAR24B_FpToSps( int Fp, double *Sps);

FUNCTYPE( VOID) PAR24B_Start( void);
FUNCTYPE( VOID) PAR24B_Stop( void);

FUNCTYPE(  INT) PAR24B_IsReady( void);
FUNCTYPE( UINT) PAR24B_GetData( long *Values, unsigned int Nvalues);
FUNCTYPE( VOID) PAR24B_FifoSizeToNvalues(

        unsigned int FifoSize,
        unsigned int *Nvalues

        );



FUNCTYPE(  INT) PAR24B_IsEmpty( void);
FUNCTYPE(  INT) PAR24B_Overflow( void);
FUNCTYPE(  INT) PAR24B_ReadFifo( long *Values);

FUNCTYPE( VOID) PAR24B_UserByte( int Value);

FUNCTYPE(  INT) PAR24B_Open( void);
FUNCTYPE(  INT) PAR24B_Close( void);
