// FILE: 24b.h                Copyright (c), Symmetric Research, 1997
//
// Header file for the PAR24B specific portion of the Scope program includes
//
//	C24bApp		main app    PAR24B specific version of CScopeApp
//	C24bWin		main window PAR24B specific version of CScopeWindow
//	CHardwareDlg	dialog for  PAR24B hardware options
//	CChannelDlg	dialog for  PAR24B channel options
//	CAcquireDlg	dialog for  PAR24B acquisition options
//
//



// SPECIFIC RESOURCE CONSTANTS

#include "24b.rch"



// GENERAL APPLICATION CONSTANTS

#include "scope.h"





//***********************************************************************


class C24bApp : public CScopeApp {
	
public:

	// Data

	int   HardPortMode;
	int   HardPortAddr;
	int   HardCalibration;
	int   HardFIFOsize;

	int   AcqDataCoding;
	int   AcqGain;
	int   AcqLedValue;
	
	long  *pDataArray;



	
	// Methods

	//{{AFX_VIRTUAL(C24bApp)
	public:
	virtual int  ExitInstance();
	//}}AFX_VIRTUAL

	int Par24bCorrectSps( float *sps );

	virtual int AtodMeasure( int *ival );
	virtual int AtodInit();
	virtual int AtodStart();
	virtual int AtodCheckReady();
	virtual int AtodCheckOverflow();
	virtual int AtodGetData();
	virtual int AtodDemuxData();
	virtual int AtodStop();

	virtual int AllocateDataArray( void );
	virtual int TriggerCondition( void );
	virtual int FillHdr(  void );
	virtual int WriteBytes( CFile *outfile );
	virtual int RegistrySettings( int direction, int sectid );
	virtual int InitAppData();
 	virtual int VerifyAppData();
 	virtual CScopeWindow* NewScopeWindow();

	
	//{{AFX_MSG(C24bApp)
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};





//***********************************************************************

class C24bWin : public CScopeWindow {

public:

	// Methods

	C24bWin();
	~C24bWin();


	//{{AFX_VIRTUAL(C24bWin)
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

	
	//{{AFX_MSG(C24bWin)
	afx_msg void OnHardwareDialog();
	afx_msg void OnChannelDialog();
	afx_msg void OnAcquireDialog();
	//}}AFX_MSG
	
	DECLARE_MESSAGE_MAP()
};





//***********************************************************************

class CHardwareDlg : public CScopeDialog {


public:
	CHardwareDlg(CWnd* pParent = NULL);   // standard constructor

	
	// Dialog Data
	
	//{{AFX_DATA(CHardwareDlg)
	enum { IDD = IDD_HARDWARE };
	CComboBox	m_CBO_PortAddr;
	CComboBox	m_CBO_FifoSize;
	//}}AFX_DATA

	int        m_Addr;
	int        m_Fifo;
	int        m_Index;

	// ClassWizard generated virtual function overrides
	
	//{{AFX_VIRTUAL(CHardwareDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL


protected:

	void SetComboAddress(void);
	void SetComboFifoSize(void);
	

	// Generated message map functions
	
	//{{AFX_MSG(CHardwareDlg)
	virtual BOOL OnInitDialog();
	virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
	afx_msg void OnOK();
	afx_msg void OnCancel();
	afx_msg void OnHelp();
	afx_msg void OnMeasureFIFO();
	afx_msg void OnSelchangeAddress();
	afx_msg void OnKillfocusAddress();
	afx_msg void OnSelchangeFifoSize();
	afx_msg void OnKillfocusFifoSize();
	//}}AFX_MSG
	
	DECLARE_MESSAGE_MAP()
};






//***********************************************************************


class CChannelDlg : public CScopeDialog {
	

public:
	CChannelDlg(CWnd* pParent = NULL);   // standard constructor
	~CChannelDlg();	                     // destructor

	
	// Data
	
	int  m_Nscan;
	char *pTempTitle;
	
	//{{AFX_DATA(CChannelDlg)
	enum { IDD = IDD_CHANNEL };
	CListBox  m_LSTscan;
	int       m_CurrChan;
	//}}AFX_DATA


	// ClassWizard generated virtual function overrides
	
	//{{AFX_VIRTUAL(CChannelDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL


protected:

	void UpdateChannelData( int icurr );
	void AddItem( int ikey );
	int  FindPos();
	

	// Generated message map functions
	
	//{{AFX_MSG(CChannelDlg)
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	virtual void OnCancel();
	virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
	afx_msg void OnHelp();
	afx_msg void OnBtnBack();
	afx_msg void OnBtnNext();
	afx_msg void OnChangeTxtCurr();
	afx_msg void OnKillfocusTxtCurr();
	afx_msg void OnKillfocusTxtTitle();
	afx_msg void OnSelchangeListScan();
	//}}AFX_MSG
	
	DECLARE_MESSAGE_MAP()
};




//***********************************************************************


class CAcquireDlg : public CScopeDialog {
	
public:
	CAcquireDlg(CWnd* pParent = NULL);

	// Data
	
	//{{AFX_DATA(CAcquireDlg)
	enum { IDD = IDD_ACQUIRE };
	//}}AFX_DATA

	float m_SPS;
	

	
	// Methods
	
	int  VerifySps();

	//{{AFX_MSG(CAcquireDlg)
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	virtual void OnCancel();
	virtual void OnHelp();
	virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
	virtual void OnKillfocusSps();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

