// FILE: scope.h                Copyright (c), Symmetric Research, 1997-2000
//
// Header file for generic portion of the Scope program includes
//
//      CScopeApp       main app    controls data, files, and registry
//      CScopeWindow    main window controls menus and graphics
//      CScopeDialog    base dialog provides validation + etc functions
//      CRegistryDlg    dialog for registry reset
//      COutputDlg      dialog for output options 
//      CDisplayDlg     dialog for display options 
//      CContentsDlg    dialog for help contents
//      CTopicDlg       dialog for help topics
//



//***********************************************************************


// WINDOWS CONSTANTS

#include <afxwin.h>
#include <afxext.h>     // needed for status bar


// RESOURCE CONSTANTS

#include "scope.rch"



// APPLICATION CONSTANTS


#define WM_EDITLOSTFOCUS        (WM_USER+100)

#define UNCHECKED               0
#define CHECKED                 1
#define INDETERMINATE           2

#define MASK_CHECK              0X03L
#define MASK_HIGHLIGHT          0X04L
#define MASK_FOCUS              0X08L

#define IDT_TIMER               1

#define MSPERSEC                1000
#define TICKINBUFF              4

#define HMARGIN                 10
#define VMARGIN                 5

#define REG_DIR_GET             0
#define REG_DIR_SAVE            1
#define REG_DIR_CLEAR           2

#define CS_MONO                 0
#define CS_GREEN                1
#define CS_BLUE                 2
#define CS_BROWN                3

#define FILE_NONE               0
#define FILE_ONE                1
#define FILE_SEQ                2
#define FILE_TIME               3


#define MAXTITLE                12
#define MAXSTR                  256


#define REG_SECT_APP            0
#define REG_SECT_SETTING        1
#define REG_SECT_DISP           2
#define REG_SECT_OUTPUT         3
#define REG_SECT_HARD           4
#define REG_SECT_CHAN           5
#define REG_SECT_ACQ            6
#define REG_SECT_NEXT           7

#define DLG_POS_INIT            -999


// This is defined to be the maximum for all A/D's that
// will work with scope.  Currently two are supported:
//
//    A/D        Max Channels
//   DSPA64         64
//   PAR24B          8
//   PAR4CH          4
//

#define MAXHDRCHANNEL           64




struct channel_data {
        long  channel;
        char  title[MAXTITLE];
        };


struct HDR_DATA {
        char   Id[32];
        long   Channels;
        long   RecordPts;
        long   WordSize;
        long   DataCoding;
        double SampleRate;
        double Gain;
        double FilterCoeff;
        double FilterScale;
        struct channel_data ChannelList[MAXHDRCHANNEL];
        };


// Needed for forward reference         
class CScopeWindow;             





//***********************************************************************



class CScopeApp : public CWinApp {
        
public:

        // Data

        int    DefaultColor;
        int    DefaultNblock;
        int    DefaultNdisplay;
        int    DefaultNsamples;
        int    DataPtSize;
        int    MaxChannel;
        int    MaxSample;
        int    MinAmpShift;
        int    MaxAmpShift;
        float  MinSPS;
        float  MaxSPS;
        


        int    PltShiftBy;
        int    PltNdisplay;
        int    PltFirstSample;
        int    PltFirstTrace;
        int    PltColorScheme;

        int    AcqNchannels;
        int    AcqNsamples;
        int    AcqNsamplesPlot;
        
        int    AcqFile;
        int    AcqSeqStart;
        int    AcqNblock;
        int    CurrBlock;
        
        float  AcqSPS;

        int    Iplotbuff;
        int    Nplotbuffers;

        long   Nbufferpts;
        long   NbufferptsPlot;
        long   Nbufferbytes;
        long   LastNbufferbytes;
        
        long   *pScanActive;
        int    *pScanOrder;
        char   *pScanTitle;

        double *pTMPoffset;
        long   *pDCoffset;
        long   *pDemuxArray;

        BOOL   IsRunning;
        BOOL   IsAllocated;
        BOOL   ResetPaint;
        BOOL   ROnsamples;
        BOOL   ForbidMultiple;
        BOOL   DoClipping;
        BOOL   DoStatusBar;
        BOOL   DoPlotTrace;
        BOOL   DoRegistryDelete;
        BOOL   ResetDC;
        BOOL   AutoDC;

        int    ErrCode;
        HWND   hWndPrev;
        
        CString WorkDir;
        CString CurrDir;
        CString AppTitle;
        CString AppClassName;
        CString ErrString;

        char szTemp[MAXSTR];                            

        HKEY hkeySect;

        struct HDR_DATA Hdr;


        
        // Methods

        //{{AFX_VIRTUAL(CScopeApp)
        virtual BOOL InitInstance();
        virtual int  ExitInstance();
        //}}AFX_VIRTUAL


        // Virtual A/D functions
        
        virtual int AtodMeasure( int *ival );
        virtual int AtodInit();
        virtual int AtodStart();
        virtual int AtodCheckReady();
        virtual int AtodCheckOverflow();
        virtual int AtodGetData();
        virtual int AtodDemuxData();
        virtual int AtodStop();

        
        // Virtual Buffer, File and Registry functions
                        
        virtual int ComputeActiveChannels( void );
        virtual int AllocateDataArray( void );
        virtual int TriggerCondition( void );
        virtual int FillHdr(  void );
        virtual int WriteBytes( CFile *outfile );
        virtual int RegistrySettings( int direction, int sectid );
        virtual int InitAppData();
        virtual int VerifyAppData();
        virtual CScopeWindow* NewScopeWindow();



        // Common File and Registry functions

        int  ComputeBufferSizes( void );
        int  AllocateBuffer( void );
        int  FreeScanArrays( void );
        int  SetScanDefaults( void );                   
        int  WriteHdr(  void );
        int  WriteData( void );
        int  InitFromRegistry();
        int  RegistryOpenKey( int sectid );
        int  RegistryClear( void );
        int  RegistryCommonSettings( int direction, int sectid );
        int  SaveRegLong(  const char *szName, const long  value );
        int  SaveRegFloat( const char *szName, const float value );
        int  SaveRegChar(  const char *szName, const char *value );
        int  SaveSetting(  const char *szName, const char *szValue );
        int  GetSetting(   const char *szName, char *szValue );
        int  GetRegLong(   const char *szName, long  *value, const long  defval );
        int  GetRegFloat(  const char *szName, float *value, const float defval );
        int  GetRegChar(   const char *szName, char  *value, const char *defval );
        int  ShowHelp( int titleID, int textID, CWnd *caller );
        int  LoadHelpTitles( CListBox *listbox );
        int  AddResourceString( CListBox *listbox, int titleID, int textID, int pos = -1 );
        void ShowError();
        void ExitError();
        
                        
        //{{AFX_MSG(CScopeApp)
        //}}AFX_MSG


        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CScopeWindow : public CFrameWnd {

        
public:

        // Data

        DWORD ColorBack;
        DWORD ColorAxis;
        DWORD ColorTitle;
        DWORD ColorTrace;
        
        CPen* PenBack;
        CPen* PenAxis;
        CPen* PenTitle;
        CPen* PenTrace;

        LOGFONT font;
        CFont*  CurrFont;

        int minPoint;
        int midPoint;
        int maxPoint;
        int cxChar;
        int cyChar;
        int MaxTitleLength;

        int RegistryPosTop;
        int RegistryPosLeft;
        int DisplayPosTop;
        int DisplayPosLeft;
        int OutputPosTop;
        int OutputPosLeft;
        int ContentsPosTop;
        int ContentsPosLeft;
        int TopicPosTop;
        int TopicPosLeft;
        int TopicPosBottom;
        int TopicPosRight;
        int HardwarePosTop;
        int HardwarePosLeft;
        int ChannelPosTop;
        int ChannelPosLeft;
        int AcquirePosTop;
        int AcquirePosLeft;

        int TraceStart;
        int TraceEnd;
        int TraceWidth;
        int TraceHeight;

        int *pYlast;
        int *pYaxis;
        int *pYclipmin;
        int *pYclipmax;

        long LastPtsPlot;

        POINT *pPtArray;
        POINT *pOldArray;

        
protected:
        CStatusBar m_wndStatusBar;
        CScrollBar m_wndVScroll;

        int  controlkey;
        BOOL IsDragging;
        BOOL PaintDeferred;


                
        // Methods
public:

        //{{AFX_VIRTUAL(CScopeWindow)
        virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
        virtual void PostNcDestroy();
        //}}AFX_VIRTUAL


        CScopeWindow();
        virtual ~CScopeWindow();

        void DrawBackground( CDC *pDC );
        void DrawTitles(     CDC *pDC );
        void DrawAxes(       CDC *pDC );
        void DrawTraces(     CDC *pDC );
        void FillTraces( int npoints );
        void UpdateTitleTraces();
        void UpdateColorScheme();
        void SetColors();
        void CreatePens();
        void DeletePens();
        void CreateFont();
        void DeleteFont();
        void DeletePlotArrays();
        void DeleteVerticalBuffers();
        void UpdateTextSize( CDC *pDC );
        void SetClipLimits( int height, int traceHalf );
        void SetWindowTitle();
        void SetScrollBar( int nBar );
        void GetSize( int *height, int *width );
        int  AllocatePlotBuffers( );
        int  ChangeFontSize( int newsize );
        int  ComputeMaxTitleLength( CDC *pDC );
        int  ComputePlotPositions( );
        void InitDialogPositions( );
        void GetDialogPosition(  RECT *r, int nID );
        void SaveDialogPosition( RECT *r, int nID );


        

        //{{AFX_MSG(CScopeWindow)
        virtual int  OnCreate(LPCREATESTRUCT lpCreateStruct);
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnPaint();
        afx_msg void OnDestroy();
        afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnKeyUp(  UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnTimer(UINT nIDEvent);
        afx_msg void OnRunProgram();
        afx_msg void OnStopProgram();
        afx_msg void OnExit();
        afx_msg void OnSetClip();
        afx_msg void OnSetDC();
        afx_msg void OnSetAutoDC();
        afx_msg void OnSetRegDelete();
        afx_msg void OnSetStatusBar();
        afx_msg void OnSetPlotTrace();
        afx_msg void OnRegFactoryDialog();
        afx_msg void OnDisplayDialog();
        afx_msg void OnOutputDialog();
        afx_msg void OnContentsDialog();
        afx_msg void OnAboutDialog();
        afx_msg void OnDebug();
        afx_msg void OnUpdateMenuRunning(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateMenuStopped(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateClipCheck(     CCmdUI* pCmdUI);
        afx_msg void OnUpdateAutoDCCheck(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateRegDeleteCheck(CCmdUI* pCmdUI);
        afx_msg void OnUpdatePlotTraceCheck(CCmdUI* pCmdUI);
        //}}AFX_MSG

        afx_msg LRESULT OnEnterSizeMove( WPARAM wParam, LPARAM lParam);
        afx_msg LRESULT OnExitSizeMove(  WPARAM wParam, LPARAM lParam);
        
        DECLARE_MESSAGE_MAP()
};





//***********************************************************************


class CScopeDialog : public CDialog {
        

public:                                       
        CScopeDialog(UINT nIdTemplate, CWnd* pParent = NULL);

        
        // Data

        int  DialogId;
        int  CancelId;
        BOOL DoValidate;
                
protected:

        int   CheckFocus( void );
        int   ValidateAsInt(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsHex(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsFloat( int nID, float minval, float maxval, float *theval, char *txt );
        int   SetDlgItemHex(   int nID, int   val );
        int   SetDlgItemFloat( int nID, float val );
        float GetDlgItemFloat( int nID, BOOL *lpTrans = NULL);
        BOOL  SetTextReadOnly( int nID );
        void  RestorePosition();
        void  RememberPosition();


        // Generated message map functions
        
        //{{AFX_MSG(CScopeDialog)
        afx_msg void OnActivateApp( BOOL bActive, HTASK hTask );
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};



//***********************************************************************

class CAboutDlg : public CDialog {

public:
        CAboutDlg(CWnd* pParent = NULL);   // standard constructor

        //{{AFX_DATA(CAboutDlg)
        enum { IDD = IDD_ABOUTBOX };
        //}}AFX_DATA


protected:

        // Generated message map functions
        
        //{{AFX_MSG(CAboutDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};




//***********************************************************************

class CRegistryDlg : public CScopeDialog {

public:
        CRegistryDlg(CWnd* pParent = NULL);   // standard constructor

        //{{AFX_DATA(CRegistryDlg)
        enum { IDD = IDD_REGISTRY };
        //}}AFX_DATA


        //{{AFX_VIRTUAL(CRegistryDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL

        
protected:

        // Generated message map functions
        
        //{{AFX_MSG(CRegistryDlg)
        virtual BOOL OnInitDialog();
        afx_msg void OnBtnYes();
        afx_msg void OnBtnNo();
        afx_msg void OnHelp();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};




//***********************************************************************


class COutputDlg : public CScopeDialog {
        

public:                                       
        COutputDlg(CWnd* pParent = NULL);                       // standard 
        COutputDlg(BOOL ROnsamples, CWnd* pParent = NULL);      // special

        // Dialog Data
        
        //{{AFX_DATA(COutputDlg)
        enum { IDD = IDD_OUTPUT };
        BOOL    m_ROnsamples;
        int     m_Nblock;
        int     m_Nsamples;
        int     m_FileOut;
        int     m_SeqStart;
        CString m_WorkDir;
        //}}AFX_DATA


        // Overrides
        
        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(COutputDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


                
protected:

        void InitData();
        void ComputeFileSize();


        // Generated message map functions
        
        //{{AFX_MSG(COutputDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnOptFileOut();
        afx_msg void OnKillfocusTxtNblock();
        afx_msg void OnKillfocusTxtNsample();
        afx_msg void OnKillfocusTxtSeqStart();
        afx_msg void OnKillfocusTxtWorkDir();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};





//***********************************************************************


class CDisplayDlg : public CScopeDialog {


public:
        CDisplayDlg(CWnd* pParent = NULL);   // standard constructor

        
        // Dialog Data
        
        //{{AFX_DATA(CDisplayDlg)
        enum { IDD = IDD_DISPLAY };
        CEdit   m_TXT_ShiftBy;
        CEdit   m_TXT_Ndisplay;
        CEdit   m_TXT_FirstSample;
        CEdit   m_TXT_FirstTrace;
        CStatic m_LAB_PlotBits;
        CStatic m_LAB_Nscan;
        CStatic m_LAB_LastSample;
        CButton m_CHK_Clipping;
        CButton m_CHK_AutoDC;
        CButton m_OPT_ColorMono;
        CButton m_OPT_ColorGreen;
        CButton m_OPT_ColorBrown;
        CButton m_OPT_ColorBlue;
        int     m_ShiftBy;
        int     m_Ndisplay;
        int     m_FirstSample;
        int     m_FirstTrace;
        int     m_ColorScheme;
        int     m_TraceWidth;
        BOOL    m_Clipping;
        BOOL    m_AutoDC;
        //}}AFX_DATA



        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CDisplayDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void SetPlotBitsLabel( void );
        void SetNscanLabel( void );
        void SetLastSampleLabel( void );
        void SetColorSchemeRadio( void );

        
        // Generated message map functions
        
        //{{AFX_MSG(CDisplayDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnKillfocusTxtShiftby();
        afx_msg void OnKillfocusTxtNdisplay();
        afx_msg void OnKillfocusTxtFirstsample();
        afx_msg void OnKillfocusTxtFirsttrace();
        afx_msg void OnChkClipping( );
        afx_msg void OnChkAutoDC( );
        afx_msg void OnOptColorScheme( );
        //}}AFX_MSG
        DECLARE_MESSAGE_MAP()


};





//***********************************************************************


class CContentsDlg : public CScopeDialog {

public:
        CContentsDlg(CWnd* pParent = NULL);   // standard constructor

        // Dialog Data
        
        //{{AFX_DATA(CContentsDlg)
        enum { IDD = IDD_HELPCONTENTS };
        CListBox        m_LST_topics;
        //}}AFX_DATA


        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CContentsDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL

        
protected:

        // Generated message map functions
        
        //{{AFX_MSG(CContentsDlg)
        afx_msg void OnBtnDisplay();
        afx_msg void OnHelp();
        virtual void OnCancel();
        virtual BOOL OnInitDialog();
        afx_msg void OnDblclkTopics();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CTopicDlg : public CScopeDialog {

public:

        // Data
        
        CString htitle;
        CString htext;
        int     m_TitleID;
        int     m_TextID;

        //{{AFX_DATA(CTopicDlg)
        enum { IDD = IDD_HELPTOPIC };
        CEdit   m_TXT_Topic;
        //}}AFX_DATA

        

        // Methods
        
        CTopicDlg(CWnd* pParent = NULL);                          // standard
        CTopicDlg(int TitleID, int TextID, CWnd* pParent = NULL); // new


        // ClassWizard generated virtual function overrides
        
        //{{AFX_VIRTUAL(CTopicDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        // Generated message map functions
        
        //{{AFX_MSG(CTopicDlg)
        virtual BOOL OnInitDialog();
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnSetfocusTopic();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};
