/* FILE: simp24b.c              Copyright (c), Symmetric Research, 1998-2002
 *      
 * Acquire PAR24B data with a simple text only display and disk save.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>        /* for WriteTmpFile O_xxx defines */

#ifdef SROS_LINUX
#include <sys/stat.h>
#else
#include <sys\stat.h>
#include <io.h>
#endif

#include "par24b.h"
#include "simp24b.h"



char   msg[80];
struct INI_data INI;     /* <<< see simp24b.h for data layout ... */



/* STRING HELPER:
 *
 *
 *
 */

char *Chop( char *s) {

        char *p;


        /* Chop spaces ' ' off the end of a string and add '\n'. */

        p = s;

        while ( *p )

                p++;


        if ( *--p == '\n' ) {

                while ( *--p == ' ' )

                        /* back up */ ;


                *++p = '\n';
                *++p = '\0';

                }


        return( s);
}

void DisplayInfo( char *info ) {

        if (INI.InfoDisplay == ON)
                printf( info );
}




/* REV MESSAGE:
 *
 *
 *
 */
 
void Rev( void) {

        /* Print the rev info. */

        DisplayInfo( "PAR24B simp24b : (c) SR 1999 : Rev {01/05/02}\n");

}






/* ERRORS:
 *
 * All errors except for .ini parsing errors go through these functions.
 *
 *
 */

char *UsageMsg[] = {

        "COMMAND LINE ERROR: %s                                      \n",
        "                                                            \n",
        "  simp24b file.ini     ... where a .ini file is required    \n",

        NULL,

        };

void ErrorCommandLine( char *msg) {


        /* Print the command line usage message and quit. */

        printf( Chop( UsageMsg[0]), msg                     );
        printf( Chop( UsageMsg[1])                          );
        printf( Chop( UsageMsg[2])                          );

        exit( 1);

}

#define ERROR_FATAL                     1
#define ErrorFatal( msg)                ErrorRuntime( msg, ERROR_FATAL)
#define ErrorMaybeFatal( msg)           ErrorRuntime( msg, INI.ExitOnError)

void ErrorRuntime( char *msg, int fatal) {

        FILE *ErrFp;

            char *datestr, dbuffer[16];
            char *timestr, tbuffer[16];
            time_t     current_time;
            struct tm *local_time;



        /* Get the date and time. */

        time( &current_time );
        local_time = localtime( &current_time );

        strftime( dbuffer, 16, "%m/%d/%y", local_time );
        strftime( tbuffer, 16, "%H:%M:%S", local_time );

        datestr = dbuffer;
        timestr = tbuffer;

//        date = _strdate( dbuffer);
//        time = _strtime( tbuffer);



        /* Append the error to simp24b.err. */

        if ( ErrFp = fopen( "SIMP24B.ERR", "a") ) {

                fprintf( ErrFp, "[%s] [%s] : %s\n", datestr, timestr, msg);

                   fclose( ErrFp);

                }



        /* Display the error and quit. */

        printf( "ERROR[simp24b]: %s\n", msg);




        /* Quit if a fatal error. */

        if ( fatal ) {
                
                PAR24B_Stop();
                PAR24B_Close();

                exit( 1);
                }

}

char *ErrorFifoSizeMsg[] = {

        "ERROR: %s FifoSize does not agree with hardware value           \n",
        "                                                                \n",
        "  *** Hardware FifoSize = %u bytes, correct .ini to agree ...   \n",

        NULL

        };

void ErrorFifoSize( char *IniFilename, unsigned int TrueFifoSize) {

        /* Print the FifoSize error message and quit. */

        printf( Chop( ErrorFifoSizeMsg[0]), IniFilename        );
        printf( Chop( ErrorFifoSizeMsg[1])                     );
        printf( Chop( ErrorFifoSizeMsg[2]), TrueFifoSize       );

        exit( 1);

}















/* INI DATA AND REPORTING:
 *
 *
 *
 */


void IniCompute( void) {


        /* Fill in the additional computed INI parameters. */

        PAR24B_SpsToFp( INI.SamplingRate, &INI.FilterPoint);
        PAR24B_FpToSps( INI.FilterPoint, &INI.TrueSamplingRate);

        INI.Channels          = PAR24B_CHANNELS;


        PAR24B_FifoSizeToNvalues( INI.FifoSize, &INI.Nvalues);

        INI.ChBufPts          = INI.Nvalues / INI.Channels;
        INI.AcqBufSize        = INI.Nvalues * sizeof( long);
        INI.OutputFileSize    = INI.AcqBufSize * INI.Blocks;
        INI.BufferTime        = INI.ChBufPts / INI.TrueSamplingRate;


}


char *IniReportFmt[] = {

        "; PAR24B INI DATA from [%s]:                            \n",
        "                                                        \n",
        "        PortAddress = 0x%X                              \n",
        "           PortMode = %s                                \n",
        "           FifoSize = %u                                \n",
        "                                                        \n",
        "         DataCoding = %s                                \n",
        "       SamplingRate = %f                                \n",
        "               Gain = %s                                \n",
        "        Calibration = %s                                \n",
        "                                                        \n",
        "       SystemStatus = %s                                \n",
        "      KeypressToRun = %s                                \n",
        "        ExitOnError = %s                                \n",
        "        DataDisplay = %s                                \n",
        "        InfoDisplay = %s                                \n",
        "     OutputFileMode = %s                                \n",
        "             Blocks = %lu                               \n",
        "                                                        \n",
        "                                                        \n",
        "                                                        \n",
        "; Additional items implied by these parameters:         \n",
        ";                                                       \n",
        ";          Channels = %d                                \n",
        ";           Nvalues = %u                                \n",
        ";          ChBufPts = %u                                \n",
        ";        AcqBufSize = %u bytes                          \n",
        ";    OutputFileSize = %lu bytes                         \n",
        ";                                                       \n",
        ";       FilterPoint = %d                                \n",
        ";  TrueSamplingRate = %f Hz                             \n",
        ";        BufferTime = %f sec                            \n",
        ";                                                       \n",

        NULL,

        };



void IniReport( void) {

        FILE *RptFp;

            char **fmt;

                char
                               *OnOff( int value),
                            *PortMode( int value),
                          *DataCoding( int value),
                         *Calibration( int value),
                                *Gain( int value),
                      *OutputFileMode( int value)

                      ;




        /* Generate the ASCII INI report file, simp24b.rpt. */

        RptFp = fopen( "SIMP24B.RPT", "w");

        if ( !RptFp )

                ErrorFatal( "unable to open simp24b.rpt report file");



        fmt = IniReportFmt;

        fprintf( RptFp, Chop( *fmt++),                 INI.Filename          );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++),                 INI.PortAddress       );
        fprintf( RptFp, Chop( *fmt++),       PortMode( INI.PortMode       )  );
        fprintf( RptFp, Chop( *fmt++),                 INI.FifoSize          );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++),     DataCoding( INI.DataCoding     )  );
        fprintf( RptFp, Chop( *fmt++),                 INI.SamplingRate      );
        fprintf( RptFp, Chop( *fmt++),           Gain( INI.Gain           )  );
        fprintf( RptFp, Chop( *fmt++),    Calibration( INI.Calibration    )  );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++),          OnOff( INI.SystemStatus   )  );
        fprintf( RptFp, Chop( *fmt++),          OnOff( INI.KeypressToRun  )  );
        fprintf( RptFp, Chop( *fmt++),          OnOff( INI.ExitOnError    )  );
        fprintf( RptFp, Chop( *fmt++),          OnOff( INI.DataDisplay    )  );
        fprintf( RptFp, Chop( *fmt++),          OnOff( INI.InfoDisplay    )  );
        fprintf( RptFp, Chop( *fmt++), OutputFileMode( INI.OutputFileMode )  );
        fprintf( RptFp, Chop( *fmt++),                 INI.Blocks            );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++)                                        );

        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++),                 INI.Channels          );
        fprintf( RptFp, Chop( *fmt++),                 INI.Nvalues           );
        fprintf( RptFp, Chop( *fmt++),                 INI.ChBufPts          );
        fprintf( RptFp, Chop( *fmt++),                 INI.AcqBufSize        );
        fprintf( RptFp, Chop( *fmt++),                 INI.OutputFileSize    );
        fprintf( RptFp, Chop( *fmt++)                                        );
        fprintf( RptFp, Chop( *fmt++),                 INI.FilterPoint       );
        fprintf( RptFp, Chop( *fmt++),                 INI.TrueSamplingRate  );
        fprintf( RptFp, Chop( *fmt++),                 INI.BufferTime        );
        fprintf( RptFp, Chop( *fmt++)                                        );

        fclose( RptFp);


}
char *OnOff( int value) {


        return( value ? "ON" : "OFF" );


}
char *PortMode( int value) {

        char *s;

        switch ( value ) {

                case PAR24B_PORT_MODE_BPP:     s = "BPP"; break;
                case PAR24B_PORT_MODE_EPP:     s = "EPP"; break;
                case PAR24B_PORT_MODE_ECP_BPP: s = "ECP/BPP"; break;
                case PAR24B_PORT_MODE_ECP_EPP: s = "ECP/EPP"; break;

                default:

                        s = "unknown PortMode"; break;
                }

        return( s );

}
char *DataCoding( int value ) {

        char *s;

        switch ( value ) {

                case PAR24B_DC_SIGNED: s = "SIGNED"; break;
                case PAR24B_DC_OFFSET: s = "OFFSET"; break;

                default:

                        s = "unknown DataCoding"; break;
                }

        return( s );

}
char *Gain( int value ) {

        char *s, *gtable[] = { "1", "2", "4", "8", "16", "32", "64", "128" };


       if ( 0 <= value && value <= 7 )

                s = gtable[ value];

       else
                s = "unknown Gain";


       return( s );


}
char *Calibration( int value ) {

        char *s;

        switch ( value ) {

                case PAR24B_CAL_MODE_SELF: s = "SELF"; break;
                case PAR24B_CAL_MODE_SOIG: s = "SOIG"; break;
                case PAR24B_CAL_MODE_NONE: s = "NONE"; break;

                default:

                        s = "unknown Calibration"; break;
                }

        return( s);

}
char *OutputFileMode( int value ) {

        char *s;

        switch ( value ) {

                case NONE:         s = "NONE";        break;
                case SINGLE:       s = "SINGLE";      break;
                case SEQUENTIAL:   s = "SEQUENTIAL";  break;
                case TIME:         s = "TIME";        break;

                default:

                        s = "unknown OutputFileMode value"; break;
                }

        return( s );

}







/* HDR DATA AND REPORTING:
 *
 *
 *
 */

struct HDR_data HDR;  /* <<< see simp24b.h for data layout ... */


void HdrReport( void ) {

        FILE *HdrFp; int i;



        /* Generate the binary HDR report file, simp24b.hdr. */

        strcpy( HDR.Id, "SR PAR24B");

        HDR.Channels       = INI.Channels;
        HDR.RecordPts      = INI.ChBufPts * INI.Blocks;
        HDR.WordSize       = 4;
        HDR.DataCoding     = INI.DataCoding;
        HDR.SamplingRate   = INI.TrueSamplingRate;
        HDR.Gain           = 1 << INI.Gain;
        HDR.FilterCoeff    = 0.0;
        HDR.FilterScale    = 1.0;


        for ( i = 0 ; i < PAR24B_CHANNELS ; i++ ) {

                HDR.ChannelList[i].channel = i;

                sprintf( HDR.ChannelList[i].title, "CH %d", i);

                }




        /* Write the .hdr file. */

        HdrFp = fopen( "SIMP24B.HDR", "wb" );

        if ( !HdrFp )

                ErrorFatal( "unable to open simp24b.hdr ..." );

        if ( fwrite( &HDR, sizeof( HDR ), 1, HdrFp ) != 1 )

                ErrorFatal( "unable to write simp24b.hdr ..." );


        fclose( HdrFp );


}






/* START MESSAGE:
 *
 *
 *
 */

char *StartMsgFmt[] = {

        "Initializing PAR24B with: (see simp24b.rpt for more details)    \n",
        "                                                                \n",
        "          PortAddress = 0x%X                                    \n",
        "             PortMode = %s                                      \n",
        "             FifoSize = %u                                      \n",
        "           DataCoding = %s                                      \n",
        "     TrueSamplingRate = %f Hz                                   \n",
        "                 Gain = %s                                      \n",
        "                                                                \n",

        NULL,

        };



void StartMsg( void ) {

        char **fmt;

        if (INI.InfoDisplay == OFF)
                return;


        /* Display a few of the initialization parameters. */

        fmt = StartMsgFmt;

            printf( Chop( *fmt++ )                                       );
            printf( Chop( *fmt++ )                                       );
            printf( Chop( *fmt++ ),              INI.PortAddress         );
            printf( Chop( *fmt++ ),    PortMode( INI.PortMode         )  );
            printf( Chop( *fmt++ ),              INI.FifoSize            );
            printf( Chop( *fmt++ ),  DataCoding( INI.DataCoding       )  );
            printf( Chop( *fmt++ ),              INI.TrueSamplingRate    );
            printf( Chop( *fmt++ ),        Gain( INI.Gain             )  );
            printf( Chop( *fmt++ )                                       );

}






/* KEYBOARD FUNCTIONS:
 *
 *
 *
 */

void KeyWait( void ) {

        int c;


        /* Wait for a keypress, quitting if requested. */

        c = GetKeyWait( );

        if ( c == 'q' || c == 'Q' )

                { PAR24B_Stop( ); PAR24B_Close( ); exit( 0 ); }


}
void KeyCheck( void ) {

        int c;


        /* When running interactively, check for a keypress. */

        if ( INI.KeypressToRun == ON ) {

                c = GetKeyCheck( );

                if ( c == 'q' || c == 'Q' )

                        { PAR24B_Stop( ); PAR24B_Close( ); exit( 0 ); }
                }
}








/* SCREEN DATA DISPLAY:
 *
 * Print out acquired data values to the screen.
 *
 *
 *
 *
 */

void DisplayValues( long *values ) {

        int i;


        for ( i = 0 ; i < INI.Nvalues ; i += 8 )

#define PRNT_FMT  "    %08lX %08lX %08lX %08lX %08lX %08lX %08lX %08lX\n"

                printf( PRNT_FMT,

                        values[i+0],
                        values[i+1],
                        values[i+2],
                        values[i+3],
                        values[i+4],
                        values[i+5],
                        values[i+6],
                        values[i+7]

                        );

        printf( "\n");
}











/* DISK FUNCTIONS:
 *
 * These functions handle writing binary data output files.
 *
 *
 */

#ifdef SROS_LINUX
#define O_BINARY O_FSYNC
#endif

#define OFLAG           (O_BINARY|O_RDWR)
#define PMODE           (S_IREAD|S_IWRITE)

void WriteTmpFile( long *buffer, int nbytes, unsigned long block ) {

        int OutputFh, oflag;



        /* Set the open flag to create or append depending on the block. */

        if ( block == 0 )

                oflag = OFLAG | (O_CREAT|O_TRUNC);

        else
                oflag = OFLAG | O_APPEND;





        /* Write the .tmp output file. */

        OutputFh = open( "SIMP24B.TMP", oflag, PMODE );

        if ( OutputFh == -1 )

                ErrorMaybeFatal( "unable to open simp24b.tmp" );

        if ( write( OutputFh, buffer, nbytes ) != nbytes )

                ErrorMaybeFatal( "simp24b.tmp write failed" );

        if ( _commit( OutputFh ) != 0 )

                ErrorMaybeFatal( "unable to commit simp24b.tmp to disk" );

        if ( close( OutputFh ) != 0 )

                ErrorMaybeFatal( "unable to close simp24b.tmp" );


}
void RenameTmpFile( int OutputFileMode, unsigned long nfile ) {

        char FinalName[16];



        /* Compute the final .out file name. */

        if ( OutputFileMode == SINGLE )

                sprintf( FinalName, "SIMP24B.OUT" );

        else if ( OutputFileMode == SEQUENTIAL )

                sprintf( FinalName, "%08lX.OUT", nfile );

        else if ( OutputFileMode == TIME )

                sprintf( FinalName, "%08lX.OUT", time( NULL ) );

        else /* OutputFileMode == NONE */

                return;



#ifdef SROS_LINUX

        /* Verify that new name is available. */

        if ( access( FinalName, F_OK) == 0 ) {

                sprintf( msg, "Rename failed.  File %s already exists.",
                                                                   FinalName);
                ErrorMaybeFatal( msg);
                }
#endif


        /* Rename the .tmp file. */

        if ( rename( "SIMP24B.TMP", FinalName ) == 0 ) {

                sprintf( msg, "simp24b.tmp renamed to %s ... \n", FinalName );
                DisplayInfo( msg );
                }

        else
                ErrorMaybeFatal( "simp24b.tmp rename failed" );


}
void SaveToDisk( long *PC_BUFFER, unsigned long nbuffer ) {

        static unsigned long block = 0;
        static unsigned long nfile = 0;


        /* If no disk save, just return. */

        if ( INI.OutputFileMode == NONE )

                { printf( "Buffer[%08lX] processed\n", nbuffer ); return; }



        /* Otherwise, save to the .tmp file ... */

        WriteTmpFile( PC_BUFFER, INI.AcqBufSize, block );

        sprintf( msg, "Buffer[%08lX] block %02lu -> simp24b.tmp\n",
                                                             nbuffer, block );
        DisplayInfo( msg );


        /* Increment the block count and check for reset. */

        if ( ++block >= INI.Blocks ) {

                RenameTmpFile( INI.OutputFileMode, nfile++);

                block = 0;

                }

}






/* SLEEP FUNCTION:
 *
 *
 *
 *
 */

int SrSleep( int ms ) {

        /* Sleep for specifed number of milliseconds.
         * Returns 0 for success, -1 for error.
         *
         * Linux nanosleep takes structure with seconds and nanoseconds.
         * No sleep function is available for MSDOS, so return immediately.
         */

#ifdef SROS_LINUX
        
        struct timespec twait, tleft;
        tleft.tv_sec  = tleft.tv_nsec = 0;
        twait.tv_sec  = (long)(ms / 1000);
        twait.tv_nsec = (ms - (twait.tv_sec*1000)) * 1000000;
        return( nanosleep( &twait, &tleft ) );

#elif defined( SROS_MSDOS )
        
        return( 0 );

#else        
        _sleep( ms );
        return( 0 );
#endif
        
}










/* MAIN:
 *
 *
 *
 *
 */

int main( int argc, char *argv[]) {

        int status, sleepms; unsigned int TrueFifoSize; long HI7190_CrValue;

            long *PC_BUFFER; unsigned long nbuffer;


        int LedValue;
                


#ifdef SROS_LINUX

        /* Allow getchar to return even if no character is waiting. */

        if ( SetNonBlock(1) == -1) 
                ErrorMaybeFatal( "Could not set non-blocking terminal read." );

#endif



        /* Check the number of command line arguments. */

        if ( argc != 2 )

                ErrorCommandLine( "wrong number of command line arguments");




        /* Setup default INI parameters. */

        INI.Filename        = argv[1];

        INI.PortAddress     = PAR24B_PORT_ADDRESS_1;
        INI.PortMode        = PAR24B_PORT_MODE_BPP,
        INI.FifoSize        = PAR24B_FIFO_SIZE_TYP;
        INI.DataCoding      = PAR24B_DC_SIGNED,
        INI.SamplingRate    = 30.0;
        INI.Gain            = PAR24B_GAIN_1,
        INI.Calibration     = PAR24B_CAL_MODE_SELF,

        INI.SystemStatus    = ON;
        INI.KeypressToRun   = ON;
        INI.ExitOnError     = ON;
        INI.DataDisplay     = ON;
        INI.InfoDisplay     = ON;
        INI.OutputFileMode  = SINGLE;
        INI.Blocks          = 2;



        /* Parse the user's .ini file. */

        ProcessIni();
                

        /* Display the rev. */

        Rev();



        /* Fill in the additional computed INI parameters ... */

        IniCompute();




        /* Generate the .rpt and .hdr files. */

        DisplayInfo( "Generating simp24b.rpt ... \n");

                IniReport();

        DisplayInfo( "Generating simp24b.hdr ... \n");

                HdrReport();

        DisplayInfo( "\n");







        /* Initialize the PAR24B. */

        StartMsg();

#define INIT_SUCCESS   "PAR24B_Init successful, HI7190 Cr[0-7] = 0x%08lX\n"
#define INIT_FAILURE   "PAR24B_Init failed, check parallel port settings"
#define INIT_SKIPPED   "PAR24B_Init skipped, SystemStatus = OFF\n"
                
        if ( INI.SystemStatus == ON ) {

                status = PAR24B_Init(

                                INI.PortAddress,
                                INI.PortMode,

                                &TrueFifoSize,

                                INI.DataCoding,
                                INI.FilterPoint,
                                INI.Gain,
                                INI.Calibration,

                                &HI7190_CrValue,

                                NULL

                                );

                if ( status == 1 ) {

                        sprintf( msg, INIT_SUCCESS, HI7190_CrValue);
                        DisplayInfo( msg );
                        }

                else
                        ErrorFatal( INIT_FAILURE);



                /* Turn off the PAR24B User Byte yellow LED. */

                PAR24B_UserByte( LedValue = 0 );

                }

        else
                DisplayInfo( INIT_SKIPPED);


        DisplayInfo( "\n");





        /* Check PAR24B_Init TrueFifoSize against INI.FifoSize. */

        if ( INI.SystemStatus == ON ) {

                if ( TrueFifoSize != INI.FifoSize )

                        ErrorFifoSize( INI.Filename, TrueFifoSize);
                }






        /* Allocate PC acquisition buffer memory:
         *
         * This PC memory is used to store data on its way out from the
         * PAR24B to the disk and screen.
         *
         *
         */

        PC_BUFFER = (long *)malloc( INI.AcqBufSize);

        if ( !PC_BUFFER )

                ErrorFatal( "unable to malloc PC buffer memory");





        /* Initialize the number of buffers processed count. */

        nbuffer = 0;


        
        /* Initialize amount of sleep time as  */
        /* (buffertime sec / 10) * 1000 ms/sec */

        sleepms = INI.BufferTime*100; 




        /* Start executing:
         *
         * The PAR24B executes continuously, saving data in its FIFO.
         *
         * Note that the FIFO is drained when it is half full so acquisition
         * continues uninterrupted while the PC saves to disk.
         *
         *
         */

        if ( INI.KeypressToRun == ON ) {

                printf( "Hit ENTER to start, Ctrl+C to quit ... \n");

                KeyWait();

                printf( "\n");

                }



        if ( INI.SystemStatus == ON )

                PAR24B_Start();









        /* Poll to see if the PAR24B is ready.
         *
         * Everytime data is ready, IsReady will return a non zero value.
         *
         *
         *
         */

#define NOT_READY       0

        sprintf(msg,"Waiting %4.2f sec for first buffer ...\n",INI.BufferTime);
        DisplayInfo( msg );

polling_loop:


        /* Is data ready? */

        if ( INI.SystemStatus == ON ) {

                status = PAR24B_IsReady();

                if ( status == NOT_READY )

                        { KeyCheck(); SrSleep( sleepms); goto polling_loop; }
                }




        /* Data is ready, copy the PAR24B buffer to PC memory. */

        if ( INI.SystemStatus == ON )

                PAR24B_GetData( PC_BUFFER, INI.Nvalues);



        /* Twiddle the yellow LED. */

        PAR24B_UserByte( LedValue ^= 1 );



        /* Save the buffer to the hard disk. */

        SaveToDisk( PC_BUFFER, nbuffer);



        /* Display the acquired data. */

        if ( INI.DataDisplay == ON )

                DisplayValues( PC_BUFFER);



        /* Increment the number of buffers processed. */

        nbuffer++;



        /* Has overflow occurred ? */

        if ( (INI.SystemStatus == ON) && PAR24B_Overflow() ) {

                ErrorMaybeFatal( "FIFO overflow during PC processing ...");
                
                /* If error was not fatal, reset and continue */
                
                PAR24B_Stop();
                PAR24B_Start();
                }

                

        /* Check for a 'q' to quit, and then continue polling. */

        KeyCheck();

    goto polling_loop;


    return( 0 );
}

















/* INI FILE PARSING:
 *
 * These functions parse the .ini input file.
 *
 * They are secondary to the main flow of the program.  Study main() above
 * for the core acquisition steps.
 *
 * These functions are more complicated than simple fscanf to allow for
 * free format syntax and better error reporting.
 *
 *
 *
 *
 */

FILE *IniFp;

int
        fline = 1;



#define MAXTOKEN        256             /* token info */

char TOKEN[ MAXTOKEN], *token;

#define SYMBOL          1               /* token types */
#define EQUALS          2
#define HEX             3
#define INTEGER         4
#define FLOAT           5
#define STRING          6
#define UNKNOWN         7

int
        ttype;




void ErrorIni( char *msg) {


        /* Display an .ini error and quit. */

        printf( "ERROR[%s] : line %d : %s", INI.Filename, fline, msg );

        exit( 1);


}
void AddToToken( int c) {


        /* Add c to TOKEN, checking for token buffer overflow. */

        if ( (token - TOKEN) < MAXTOKEN )

                *token++ = c;

        else
                ErrorIni( "internal token length exceeded");


}
int SkipSpace( int c) {


        /* Skip over white space, including comments. */
skip:

        if ( isspace( c) ) {

                if ( c == '\n' )

                        fline++;
                     

                c = fgetc( IniFp);

                goto skip;

                }


        else if ( c == ';' ) { /* ; comment ... */

                while ( c != '\n' && c != EOF )

                        c = fgetc( IniFp);

                goto skip;

                }


        return( c);


}
int GetToken( void) {

        static int c = ' ';



        /* Skip over any white space. */

        c = SkipSpace( c);



        /* Gather the next token. */

        token = TOKEN;


        if ( isalpha( c) || c == '_' ) {

                ttype = SYMBOL;

                while ( isalnum( c) || c == '_' || c == '/' )

                        { AddToToken( c); c = fgetc( IniFp); }
                }


        else if ( c == '=' ) {

                ttype = EQUALS;

                  AddToToken( c); c = fgetc( IniFp);

                }


        else if ( isdigit( c) ) {

                /* ttype could be HEX, INTEGER, or FLOAT ... */

                AddToToken( c);

                  c = fgetc( IniFp);



                if ( TOKEN[0] == '0' && c == 'x' ) {

                        ttype = HEX;
                        token = TOKEN;

                        c = fgetc( IniFp);

                        if ( isxdigit( c) ) {

                                while ( isxdigit( c) )

                                        { AddToToken( c); c = fgetc( IniFp); }
                                }

                        else

                                ttype = UNKNOWN;
                        }



                else {

                        ttype = INTEGER;

                        while ( isdigit( c) )

                                { AddToToken( c); c = fgetc( IniFp); }



                        if ( c == '.' ) {

                                ttype = FLOAT;

                                AddToToken( c);

                                  c = fgetc( IniFp);

                                while ( isdigit( c) )

                                        { AddToToken( c); c = fgetc( IniFp); }
                                }
                        }
                }



        else if ( c == '"' ) {

                ttype = STRING;

                c = fgetc( IniFp);

                  while ( c != '"' && c != '\n' && c != EOF )

                        { AddToToken( c); c = fgetc( IniFp); }



                if ( c == '"' )

                        c = fgetc( IniFp);

                else
                        ErrorIni( "missing quote on string");

                }



        else if ( c == EOF )

                ttype = EOF;


        else {

                ttype = UNKNOWN;

                  AddToToken( c);


                c = fgetc( IniFp);

                }




        /* Null terminate the TOKEN and return its type. */

        AddToToken( '\0');

        return( ttype);




}
int Match( char *s) {

        char *t;


        /* Check for a match between s and TOKEN. */

        t = TOKEN;

        while ( *s && *t && toupper( *s) == toupper( *t) )

                { s++; t++; }


        return ( *s == '\0' && *t == '\0' ? 1 : 0 );



}
int KeywordEquals( char *keyword) {

        char msg[80];

#define KEYWORD_EQUALS_ERROR   "expected = sign after %s keyword"



        if ( Match( keyword) ) {


                if ( GetToken() == EQUALS )

                        return( 1);

                else {
                        sprintf( msg, KEYWORD_EQUALS_ERROR, keyword);

                          ErrorIni( msg);

                        }
                }



        return( 0);


}
void GetOnOff( int *parameter, char *error) {


        /* Get an {ON,OFF} parameter value. */

        GetToken();

        if ( Match( "ON") )

                *parameter = ON;

        else if ( Match( "OFF") )

                *parameter = OFF;

        else
                ErrorIni( error);


}
void GetInt( int *parameter, char *error) {


        /* Get a decimal int parameter value. */

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error);


        sscanf( TOKEN, "%d", parameter);


}
void GetUint( unsigned int *parameter, char *error) {


        /* Get a decimal int parameter value. */

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error);


        sscanf( TOKEN, "%u", parameter);


}
void GetUlong( unsigned long *parameter, char *error) {

        /* Get an unsigned long parameter value. */

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error);


        sscanf( TOKEN, "%lu", parameter);


}
void GetHex( int *parameter, char *error) {

        /* Get a hex parameter value. */

        GetToken();

        if ( ttype != HEX )

                ErrorIni( error);


        sscanf( TOKEN, "%x", parameter);



}
void GetFloat( double *parameter, char *error) {


        /* Get a floating point parameter value. */

        GetToken();

        if ( (ttype != FLOAT) && (ttype != INTEGER) )

                ErrorIni( error);


        sscanf( TOKEN, "%lf", parameter);


}
int ConvertGain( int value) {

        int lnvalue, gtable[] = { 1, 2, 4, 8, 16, 32, 64, 128 };



        /* Compute the log of the gain value. */

        for ( lnvalue = 0 ; lnvalue <= 7 ; lnvalue++ ) {

                if ( gtable[ lnvalue] == value )

                        return( lnvalue);
        
                }


        return( -1);


}
int ValidateFifoSize( unsigned int value) {

        int i;

            unsigned int SizeTable[] =

#if defined( SROS_MSDOS )
              { 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 0 };
#else
              { 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 0 };
#endif // SROS_MSDOS



        /* Check to see if the value is one of the allowed sizes. */

        for ( i = 0 ; SizeTable[i] ; i++ ) {

                if ( value == SizeTable[i] )

                        break;
                }



        return( SizeTable[i] ? 1 : 0 );


}
void ParseIni( void) {

        int Gain; unsigned int FifoSize;



        /* Parse the ini file. */

parse:
        GetToken();



        if ( KeywordEquals( "PortAddress") ) {

                GetHex( &INI.PortAddress, "illegal PortAddress hex value");

                if ( INI.PortAddress < 0x100 || 0x3FF < INI.PortAddress )

                        ErrorIni( "portAddress must be 0x100 to 0x3FF");

                }



        else if ( KeywordEquals( "PortMode") ) {

                GetToken();


                if ( Match( "BPP") )

                        INI.PortMode = PAR24B_PORT_MODE_BPP;

                else if ( Match( "EPP") )

                        INI.PortMode = PAR24B_PORT_MODE_EPP;

                else if ( Match( "ECP/BPP") )

                        INI.PortMode = PAR24B_PORT_MODE_ECP_BPP;

                else if ( Match( "ECP/EPP") )

                        INI.PortMode = PAR24B_PORT_MODE_ECP_EPP;

                else
                        ErrorIni( "PortMode must = {BPP,EPP,ECP/BPP,ECP/EPP}" );

                }



        else if ( KeywordEquals( "FifoSize") ) {

                GetUint( &FifoSize, "illegal FifoSize integer value");


                if ( ValidateFifoSize( FifoSize) )

                        INI.FifoSize = FifoSize;

                else
                        ErrorIni( "FifoSize must be 2^n (4096,32768 typical)");

                }



        else if ( KeywordEquals( "DataCoding") ) {

                GetToken();


                if ( Match( "SIGNED") )

                        INI.DataCoding = PAR24B_DC_SIGNED;

                else if ( Match( "OFFSET") )

                        INI.DataCoding = PAR24B_DC_OFFSET;

                else
                        ErrorIni( "DataCoding must = {SIGNED,OFFSET}");

                }



        else if ( KeywordEquals( "SamplingRate") ) {

                GetFloat( &INI.SamplingRate, "SamplingRate must = float");

                if ( INI.SamplingRate < 9.54 || 1953.00 < INI.SamplingRate )

                        ErrorIni( "SamplingRate must be 9.54 to 1953.00 Hz");

                }



        else if ( KeywordEquals( "Gain") ) {

                GetInt( &Gain, "illegal Gain integer value");

                INI.Gain = ConvertGain( Gain);

                if ( INI.Gain == -1 )

                      ErrorIni( "Gain must = {1,2,4,8,16,32,64,128}");

                }



        else if ( KeywordEquals( "Calibration") ) {

                GetToken();


                if ( Match( "SELF") )

                        INI.Calibration = PAR24B_CAL_MODE_SELF;

                else if ( Match( "SOIG") )

                        INI.Calibration = PAR24B_CAL_MODE_SOIG;

                else if ( Match( "NONE") )

                        INI.Calibration = PAR24B_CAL_MODE_NONE;

                else
                        ErrorIni( "Calibration must = {SELF,SOIG,NONE}");

                }



        else if ( KeywordEquals( "SystemStatus") )

                GetOnOff( &INI.SystemStatus, "SystemStatus must = {ON,OFF}");


        else if ( KeywordEquals( "KeypressToRun") )

                GetOnOff( &INI.KeypressToRun, "KeypressToRun must = {ON,OFF}");


        else if ( KeywordEquals( "ExitOnError") )

                GetOnOff( &INI.ExitOnError, "ExitOnError must = {ON,OFF}");


        else if ( KeywordEquals( "DataDisplay") )

                GetOnOff( &INI.DataDisplay, "DataDisplay must = {ON,OFF}");


        else if ( KeywordEquals( "InfoDisplay") )

                GetOnOff( &INI.InfoDisplay, "InfoDisplay must = {ON,OFF}");


        else if ( KeywordEquals( "OutputFileMode") ) {

                GetToken();


                if ( Match( "NONE") )

                        INI.OutputFileMode = NONE;

                else if ( Match( "SINGLE") )

                        INI.OutputFileMode = SINGLE;

                else if ( Match( "SEQUENTIAL") )

                        INI.OutputFileMode = SEQUENTIAL;

                else if ( Match( "TIME") )

                        INI.OutputFileMode = TIME;

                else

                    ErrorIni(

                        "OutputFileMode must = {NONE,SINGLE,SEQUENTIAL,TIME}"

                            );
                }


        else if ( KeywordEquals( "Blocks") )

                GetUlong( &INI.Blocks, "illegal Blocks integer value");


        else if ( ttype == EOF )

                return;



        else /* token unrecognized ... */

                ErrorIni( "unknown simp24b.ini syntax");




        /* Continue parsing. */

        goto parse;



}
int ProcessIni( void) {


        /* Open and parse the .ini file. */

        IniFp = fopen( INI.Filename, "r");

        if ( !IniFp )

                ErrorCommandLine( "unable to open .ini file");



        ParseIni();

        fclose( IniFp);



        /* Run any final .ini parameter checks. */

                /* ... none to do for PAR24B ... */

        return( 1 );
}
