Attribute VB_Name = "modPAR24B"
Attribute VB_Description = "Declarations of DLL procedures for high level use of SR PAR24B board"

Option Explicit


' FILE: par24b.bas                  Copyright (c), Symmetric Research, 2001
'
' Basic header file for PAR24B acquisition library functions.
'
' Visual Basic users should include this file in their projects to have
' access to the PAR24B library functions.
'




' RELEASE REV INFO:
'
' This define, variable, and function allow the user to determine
' library rev at either compile or runtime.
'
' The rev is an integer, where a number like 134 would be referred to
' as rev "1.34".
'
' Short rev history:
'
'   PAR24B_REV  230  ( 12/20/2000 )  < Toshiba BPP fix
'   PAR24B_REV  231  ( 12/15/2001 )  < ACPI, 64K FIFO, ECP support
'   PAR24B_REV  232  ( 12/25/2001 )  < DbEndPtr update fix
'   PAR24B_REV  233  ( 12/30/2001 )  < Add VB demo, convert

Public Const PAR24B_REV As Long = 233



' Keyboard helper defines

Public Const KEY_CHECK  As Long = 0
Public Const KEY_WAIT   As Long = 1





' PAR24B_Init DEFINES:
'
' Use these defines with PAR24B_Init to initialize the PAR24B board.
'
' Several of these constants map directly into HI7190 register bit
' setings.  Don't change their values!
'
' Some of the abbreviations are:
'
'       EPP              enhanced parallel port
'       BPP              bidirectional parallel port
'       ECP              extended capabilities port
'
'       DC               HI7190 data coding
'       FP               HI7190 filter point (sampling rate)
'       GAIN             HI7190 preamp gain
'       CAL_MODE         HI7190 calibration mode



' Parallel port defines ...

Public Const PAR24B_PORT_ADDRESS_1     As Long = &H378
Public Const PAR24B_PORT_ADDRESS_2     As Long = &H278

Public Const PAR24B_PORT_MODE_BPP      As Long = 0
Public Const PAR24B_PORT_MODE_EPP      As Long = 1
Public Const PAR24B_PORT_MODE_ECP_BPP  As Long = 2
Public Const PAR24B_PORT_MODE_ECP_EPP  As Long = 3



' HI7190 control register defines ...

' HI7190 control register defines ...

Public Const PAR24B_DC_OFFSET     As Long = 0
Public Const PAR24B_DC_SIGNED     As Long = 1

Public Const PAR24B_FP_MIN        As Long = 10
Public Const PAR24B_FP_MAX        As Long = 2047
Public Const PAR24B_FP_30Hz       As Long = 651     ' = &H28B ->  30.00192 Hz
Public Const PAR24B_FP_100Hz      As Long = 195     ' = &H0C3 -> 100.16025 Hz

Public Const PAR24B_GAIN_1        As Long = 0
Public Const PAR24B_GAIN_2        As Long = 1
Public Const PAR24B_GAIN_4        As Long = 2
Public Const PAR24B_GAIN_8        As Long = 3
Public Const PAR24B_GAIN_16       As Long = 4
Public Const PAR24B_GAIN_32       As Long = 5
Public Const PAR24B_GAIN_64       As Long = 6
Public Const PAR24B_GAIN_128      As Long = 7

Public Const PAR24B_CAL_MODE_NONE As Long = 0
Public Const PAR24B_CAL_MODE_SELF As Long = 1
Public Const PAR24B_CAL_MODE_SOIG As Long = 5


' PAR24B_Init ERRORCODES:
'
' PAR24B_Init returns a 1 or 0 for success or failure.  If it fails, the
' errorcode will be filled in with one of the following integers for more
' information.

Public Const PAR24B_IERROR_SUCCESS                As Long = 0
Public Const PAR24B_IERROR_PORT_ADDRESS           As Long = 1
Public Const PAR24B_IERROR_PORT_MODE              As Long = 2
Public Const PAR24B_IERROR_CANT_COMPUTE_FIFO_SIZE As Long = 3
Public Const PAR24B_IERROR_DC_OFFSET              As Long = 4
Public Const PAR24B_IERROR_FP                     As Long = 5
Public Const PAR24B_IERROR_GAIN                   As Long = 6
Public Const PAR24B_IERROR_CAL_MODE               As Long = 7
Public Const PAR24B_IERROR_HI7190_CR_READBACK     As Long = 8
Public Const PAR24B_IERROR_DRIVER_NOT_OPEN        As Long = 9


' PAR24B SYSTEM DEFINES:
'
' The PAR24B is fixed at eight channels.
'
' The min and max FIFO sizes are the extremes of commercially available
' parts in the 72XX class that can be used on the board.
'
' Use PAR24B_Init to dynamically compute the FIFO size on your particular
' board.
'
' Most PAR24B boards ship with a 4K or 32K FIFO, but 64K FIFOs can be
' purchased directly from chip distributors.  However, the DOS version
' of this software can not support the 64K FIFO due to limitations
' imposed by the segemented nature of that OS.


' PAR24B board parameter defines.

Public Const PAR24B_CHANNELS           As Long = 8

Public Const PAR24B_FIFO_SIZE_MIN      As Long = 256           ' 256 bytes
Public Const PAR24B_FIFO_SIZE_TYP      As Long = 4096          '  4K bytes
Public Const PAR24B_FIFO_SIZE_MAX      As Long = 65536         ' 64K bytes






' PAR24B CONTROL/EXECUTION FUNCTIONS:
'
' These functions control the operation of the PAR24B.
'
' Use the defines above and helper functions below for setting PAR24B_Init
' parameters.
'
' The size of the values array to use the PAR24B_GetData, Nvalues, should
' be computed from the FifoSize with PAR24B_FifoSizeToNvalues.
'
' Note that PAR24B_UserByte will not work until the board has been
' initialized with PAR24B_Init.


' PAR24B function library prototypes (Declares)
'
' Note: These symbols can replace the "As Type" info for function arguments.
' For example (ByVal X As Long, ByRef Y As Single) equals (ByVal X&, ByRef Y!)
'
' % Integer (short)
' & Long    (32-bit int or long)
' ! Single  (float)
' # Double  (double)
' $ String  (char*)  ByVal BSTR = LPSTR, ByRef BSTR = ptr to ptr to null-term string


' Note: Two parameters have been put on some lines to avoid exceeding
'       the maximum number of line continuations allowed in VB 4.0
Public Declare Function PAR24B_Init _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Init@36" _
   (ByVal PortAddress As Long, _
    ByVal PortMode As Long, _
    ByRef FifoSize As Long, _
    ByVal Dc As Long,   ByVal Fp As Long, _
    ByVal Gain As Long, ByVal CalMode As Long, _
    ByRef HI7190_CrValue As Long, _
    ByRef ErrorCode As Long) As Long



Public Declare Sub PAR24B_SpsToFp _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_SpsToFp@12" _
   (ByVal Sps As Double, _
    ByRef Fp As Long)



Public Declare Sub PAR24B_FpToSps _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_FpToSps@8" _
   (ByVal Fp As Long, _
    ByRef Sps As Double)



Public Declare Sub PAR24B_Start _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Start@0" ()



Public Declare Sub PAR24B_Stop _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Stop@0" ()



Public Declare Function PAR24B_IsReady _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_IsReady@0" () _
    As Long



Public Declare Function PAR24B_GetData _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_GetData@8" _
   (ByRef Values As Long, _
    ByVal Nvalues As Long) _
    As Long



Public Declare Sub PAR24B_FifoSizeToNvalues _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_FifoSizeToNvalues@8" _
   (ByVal FifoSize As Long, _
    ByRef Nvalues As Long)



Public Declare Function PAR24B_IsEmpty _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_IsEmpty@0" () _
    As Long



Public Declare Function PAR24B_Overflow _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Overflow@0" () _
    As Long



Public Declare Function PAR24B_ReadFifo _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_ReadFifo@4" _
   (ByRef Values As Long) _
    As Long



Public Declare Sub PAR24B_UserByte _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_UserByte@4" _
   (ByVal Value As Long)



Public Declare Function PAR24B_Open _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Open@0" () _
    As Long



Public Declare Function PAR24B_Close _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_Close@0" () _
    As Long



Public Declare Function PAR24B_GetRev _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_PAR24B_GetRev@0" () _
    As Long



Public Declare Function GetKey _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_GetKey@4" _
   (ByVal wait As Long) _
    As Long



Public Declare Function GetKeyCheck _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_GetKeyCheck@0" () _
    As Long



Public Declare Function GetKeyWait _
    Lib "C:\SR\PAR24B\DLL\PAR24B.DLL" _
    Alias "_GetKeyWait@0" () _
    As Long

